/*
 *  ScanGear MP for Linux
 *  Copyright CANON INC. 2007
 *  All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * NOTE:
 *  - As a special exception, this program is permissible to link with the
 *    libraries released as the binary modules.
 *  - If you write modifications of your own for these programs, it is your
 *    choice whether to permit this exception to apply to your modifications.
 *    If you do not wish that, delete this exception.
*/

#ifndef	_COLORADJUST_FR_C_
#define	_COLORADJUST_FR_C_


#include "coloradjust_fr.h"

	static	CNMSInt32		fr_current_channel			= CNMS_CA_CHANNEL_MASTER;	/* For Keep Current Channel 				*/
	static	CNMSFRValue		*fr_current					= CNMSNULL;					/* Selecting Channel Value					*/
	static	GdkPixmap		*fr_pixmap_drawing_area		= CNMSNULL;					/* Pixmap For Graph Drawing Area 			*/
	static	GdkGC			*fr_current_gc_drawing_area	= CNMSNULL;					/* Graphic Context For Graph Drawing Area	*/

static CNMSVoid CnmsFRToneCurveDraw(
						GtkWidget			*widget,
						CNMSInt32			color,
						CNMSInt32			channel					);

static CNMSVoid CnmsFRSetChannelData(
						CNMSFRValue			*current				);


/***   FinalReview Dialog Show   ***/
CNMSVoid CnmsFRDialogShow(
						CNMSInt32			color,
						CNMSInt32			keep					)
{
	/* Calcurate ToneCurve for FinalReview					*/
	CnmsColAdjCurveCalcFR( &ca_param_all, ca_usertone, ca_resulttone );

	/* Dialog Position Initialize */
	if( ( fr_pos.x == CNMS_ERR ) && ( fr_pos.y == CNMS_ERR ) ){
		CnmsColAdjSetDialogInitPosition( &fr_pos, dialog_final_review );
	}

	/* Channel ComboBox Item Change depending on ColorGroup */
	switch( color )
	{
		case CNMS_CA_COLORGROUP_COLOR:
				CnmsColAdjComboBoxItemAllClear( GTK_COMBO_BOX( fr_combo_box ) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( fr_combo_box ), gettext(ca_ch_litem_master) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( fr_combo_box ), gettext(ca_ch_litem_red) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( fr_combo_box ), gettext(ca_ch_litem_green) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( fr_combo_box ), gettext(ca_ch_litem_blue) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_a1 ), gettext(ca_label_red) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_b1 ), gettext(ca_label_red) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_a2 ), gettext(ca_label_green) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_b2 ), gettext(ca_label_green) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_a3 ), gettext(ca_label_blue) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_b3 ), gettext(ca_label_blue) );
				CnmsFRPreviewRGBKValueUnDisp();
				ca_current_colorgroup_num = CNMS_CA_COLORGROUP_COLOR;
				break;
		case CNMS_CA_COLORGROUP_GRAY:
				CnmsColAdjComboBoxItemAllClear( GTK_COMBO_BOX( fr_combo_box ) );
				gtk_combo_box_append_text( GTK_COMBO_BOX( fr_combo_box ), gettext(ca_ch_litem_gray) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_a1 ), gettext(ca_label_gray) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_b1 ), gettext(ca_label_gray) );
				gtk_label_set_text( GTK_LABEL( fr_label_str_a2 ), ca_label_null );
				gtk_label_set_text( GTK_LABEL( fr_label_str_b2 ), ca_label_null );
				gtk_label_set_text( GTK_LABEL( fr_label_str_a3 ), ca_label_null );
				gtk_label_set_text( GTK_LABEL( fr_label_str_b3 ), ca_label_null );
				CnmsFRPreviewRGBKValueUnDisp();
				ca_current_colorgroup_num = CNMS_CA_COLORGROUP_GRAY;
				break;
		case CNMS_CA_COLORGROUP_MONO:
				if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 )
					CnmsColAdjDialogHide( dialog_final_review, &fr_pos );
				return;
		default:
				DBGMSG( "[CnmsFRDialogShow] Arg Value is invalid. color[%d]\n",color );
				return;
	}

	/* Show/UnShow Change */
	if( keep == CNMS_CA_DIALOG_NON_KEEP )
	{
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) == 0 ){
			CnmsColAdjDialogShow( dialog_final_review, &fr_pos );
			gtk_combo_box_set_active( GTK_COMBO_BOX( fr_combo_box ), fr_current_channel );
		}
		else{
			CnmsColAdjDialogHide( dialog_final_review, &fr_pos );
		}
	}
	else{
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
			gtk_combo_box_set_active( GTK_COMBO_BOX( fr_combo_box ), 0 );
		}
	}

	return;
}


/***   Reset   ***/
CNMSVoid CnmsFRReset(
						CNMSBool			histFlag				)
{
	CNMSInt32	i = 0;

	/* Channel Set ComboBox Top				*/
	fr_current_channel = 0;
	if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
		gtk_combo_box_set_active( GTK_COMBO_BOX( fr_combo_box ), fr_current_channel );
	}

	/* Initialize Histogram	Value			*/
	if( histFlag == TRUE ){
		for( i=0 ; i<CNMS_CA_VECTOR_LEN ; i++ ){
			fr_master.hist[i] = fr_red.hist[i] = fr_green.hist[i] = fr_blue.hist[i] = fr_gray.hist[i] = 0;
		}
		CnmsFRToneCurveDrawFromOtherParamChanged();
	}
	
	return;
}


/***   Channel ComboBox Changed   ***/
CNMSVoid CnmsFRChannelComboBoxChanged(
						GtkComboBox			*combobox				)
{
	CNMSInt8	*channel	= CNMSNULL;

	/* If flag is True, Called by Combobox Clear func. The case, No action the process  */
	if( ca_combobox_clear_flag == CNMS_TRUE ){
		return;
	}

	if( combobox == CNMSNULL ){
		DBGMSG( "[CnmsFRChannelComboBoxChanged] Arg Pointer[combobox] is NULL.\n" );
		return;
	}
		
	/* Get Current Channel as String */
	if( ( channel = gtk_combo_box_get_active_text( combobox ) ) == CNMSNULL ){
		DBGMSG( "[CnmsBCChannelComboBoxChanged] Pointer[channel] is NULL.\n" );
		return;
	}

	if		( CnmsStrCompare( channel, gettext( ca_ch_litem_master ) ) == CNMS_NO_ERR ){
		fr_current_channel = CNMS_CA_CHANNEL_MASTER;									/* Convert Current Channel to Number from String	*/
		CnmsFRSetChannelData( &fr_master );												/* Set Data & ToneCurve Current Channel				*/
	}
	else if	( CnmsStrCompare( channel, gettext( ca_ch_litem_red ) )    == CNMS_NO_ERR ){
		fr_current_channel = CNMS_CA_CHANNEL_RED;
		CnmsFRSetChannelData( &fr_red );
	}
	else if	( CnmsStrCompare( channel, gettext( ca_ch_litem_green ) )  == CNMS_NO_ERR ){
		fr_current_channel = CNMS_CA_CHANNEL_GREEN;
		CnmsFRSetChannelData( &fr_green );
	}
	else if	( CnmsStrCompare( channel, gettext( ca_ch_litem_blue ) )   == CNMS_NO_ERR ){
		fr_current_channel = CNMS_CA_CHANNEL_BLUE;
		CnmsFRSetChannelData( &fr_blue );
	}
	else if	( CnmsStrCompare( channel, gettext( ca_ch_litem_gray ) )   == CNMS_NO_ERR ){
		fr_current_channel = CNMS_CA_CHANNEL_GRAY;
		CnmsFRSetChannelData( &fr_gray );
	}
	else{
		DBGMSG( "[CnmsFRChannelComboBoxChanged] Channel string[%s] is invalid.\n",channel );
	}
		
	return;
}


/***   Event for Drawing Graph   ***/
CNMSVoid CnmsFRDrawingAreaConfigureEvent(
						GtkWidget			*widget,
						GdkEventConfigure	*event					)
{
	if( ( widget == CNMSNULL ) || ( event == CNMSNULL ) ){
		DBGMSG( "[CnmsFRDrawingAreaConfigureEvent] Arg Pointer is NULL. widget[%p], event[%p]\n", widget, event );
		return;
	}

	/* Create Pixmap			*/
	if ( fr_pixmap_drawing_area != CNMSNULL ){
		gdk_pixmap_unref( fr_pixmap_drawing_area );
	}
	fr_pixmap_drawing_area = gdk_pixmap_new( widget->window, widget->allocation.width, widget->allocation.height, -1 );

	/* Create Graphic Context	*/
	if( fr_current_gc_drawing_area == CNMSNULL )
		fr_current_gc_drawing_area = gdk_gc_new( widget->window );

	/* Draw ToneCurve			*/
	CnmsFRToneCurveDraw( fr_drawarea, ca_current_colorgroup_num, fr_current_channel );
	
	return;
}


/***   Drawing Pixmap Event for Hiden Pixmap Area   ***/
CNMSVoid CnmsFRDrawingAreaExposeEvent(
						GtkWidget			*widget,
						GdkEventExpose		*event					)
{
	if( ( widget == CNMSNULL ) || ( event == CNMSNULL ) ){
		DBGMSG( "[CnmsFRDrawingAreaExposeEvent] Arg Pointer is NULL. widget[%p], event[%p]\n", widget, event );
		return;
	}

	/* Drawing Pixmap	*/
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		fr_pixmap_drawing_area, event->area.x, event->area.y, event->area.x, event->area.y, event->area.width, event->area.height);
		
	return;
}


/***   Draw FinalReview ToneCurve from Other Parameter Changed   ***/
CNMSVoid CnmsFRToneCurveDrawFromOtherParamChanged(
						CNMSVoid									)
{
	if( dialog_final_review != CNMSNULL ){
		if( GTK_WIDGET_VISIBLE( dialog_final_review ) != 0 ){
			CnmsFRToneCurveDraw( fr_drawarea, ca_current_colorgroup_num, fr_current_channel );
		}
	}
	return;
}


/***   Disp RGBK Value   ***/
CNMSVoid CnmsFRPreviewRGBKValueDisp(
						CNMSInt32			rValA,
						CNMSInt32			rValB,
						CNMSInt32			gValA,
						CNMSInt32			gValB,
						CNMSInt32			bValA,
						CNMSInt32			bValB					)
{
	CNMSInt8		value_a1[ CNMS_CA_STRING_LEN ],
					value_b1[ CNMS_CA_STRING_LEN ],
					value_a2[ CNMS_CA_STRING_LEN ],
					value_b2[ CNMS_CA_STRING_LEN ],
					value_a3[ CNMS_CA_STRING_LEN ],
					value_b3[ CNMS_CA_STRING_LEN ];

	if( dialog_final_review == CNMSNULL ){
		DBGMSG( "[CnmsFRPreviewRGBKValueDisp] Pointer is NULL. dialog_final_review[%p]\n", dialog_final_review );
		return;
	}
	else if( ( rValA < 0 ) || ( rValB < 0 ) || ( gValA < 0 ) || ( gValB < 0 ) || ( bValA < 0 ) || ( bValB < 0 ) ){
		CnmsFRPreviewRGBKValueUnDisp();
		DBGMSG( "[CnmsFRPreviewRGBKValueDisp] Value is invalid. rValA[%d], rValB[%d], gValA[%d], gValB[%d], bValA[%d], bValB[%d]\n",
																											rValA, rValB, gValA, gValB, bValA, bValB );
		return;
	}

	switch( ca_current_colorgroup_num )
	{
		case CNMS_CA_COLORGROUP_COLOR:
				sprintf(value_a1,"%d\0",rValA);
				gtk_label_set_text( GTK_LABEL( fr_label_value_a1 ), value_a1 );
				sprintf(value_b1,"%d\0",rValB);
				gtk_label_set_text( GTK_LABEL( fr_label_value_b1 ), value_b1 );
				sprintf(value_a2,"%d\0",gValA);
				gtk_label_set_text( GTK_LABEL( fr_label_value_a2 ), value_a2 );
				sprintf(value_b2,"%d\0",gValB);
				gtk_label_set_text( GTK_LABEL( fr_label_value_b2 ), value_b2 );
				sprintf(value_a3,"%d\0",bValA);
				gtk_label_set_text( GTK_LABEL( fr_label_value_a3 ), value_a3 );
				sprintf(value_b3,"%d\0",bValB);
				gtk_label_set_text( GTK_LABEL( fr_label_value_b3 ), value_b3 );
				break;
		case CNMS_CA_COLORGROUP_GRAY:
				sprintf(value_a1,"%d\0",gValA);
				gtk_label_set_text( GTK_LABEL( fr_label_value_a1 ), value_a1 );
				sprintf(value_b1,"%d\0",gValB);
				gtk_label_set_text( GTK_LABEL( fr_label_value_b1 ), value_b1 );
				break;
		default:
			DBGMSG( "[CnmsFRPreviewRGBKValueDisp] Value is invalid. ca_current_colorgroup_num[%d]\n", ca_current_colorgroup_num );
				break;
	}
	return;
}


/***   UnDisp RGBK Value   ***/
CNMSVoid CnmsFRPreviewRGBKValueUnDisp(
						CNMSVoid									)
{
	if( dialog_final_review == CNMSNULL ){
		DBGMSG( "[CnmsFRPreviewRGBKValueUnDisp] Pointer is NULL. dialog_histogram[%p]\n", dialog_final_review );
		return;
	}
	if( ( fr_label_value_a1 == CNMSNULL ) || ( fr_label_value_a2 == CNMSNULL ) || ( fr_label_value_a3 == CNMSNULL ) ){
		DBGMSG( "[CnmsFRPreviewRGBKValueUnDisp] Pointer is NULL. fr_label_value_a1[%p], fr_label_value_a2[%p], fr_label_value_a3[%p]\n",
																	 					fr_label_value_a1, fr_label_value_a2, fr_label_value_a3 );
		return;
	}
	if( ( fr_label_value_b1 == CNMSNULL ) || ( fr_label_value_b2 == CNMSNULL ) || ( fr_label_value_b3 == CNMSNULL ) ){
		DBGMSG( "[CnmsFRPreviewRGBKValueUnDisp] Pointer is NULL. fr_label_value_b1[%p], fr_label_value_b2[%p], fr_label_value_b3[%p]\n",
																	 					fr_label_value_b1, fr_label_value_b2, fr_label_value_b3 );
		return;
	}

	gtk_label_set_text( GTK_LABEL( fr_label_value_a1 ), ca_label_null );
	gtk_label_set_text( GTK_LABEL( fr_label_value_b1 ), ca_label_null );
	gtk_label_set_text( GTK_LABEL( fr_label_value_a2 ), ca_label_null );
	gtk_label_set_text( GTK_LABEL( fr_label_value_b2 ), ca_label_null );
	gtk_label_set_text( GTK_LABEL( fr_label_value_a3 ), ca_label_null );
	gtk_label_set_text( GTK_LABEL( fr_label_value_b3 ), ca_label_null );
	
	return;
}


/***   Drawing ToneCurve depending on Channel   ***/
static CNMSVoid CnmsFRToneCurveDraw(
						GtkWidget			*widget,
						CNMSInt32			color,
						CNMSInt32			channel					)
{
	if( ( widget == CNMSNULL ) || ( fr_pixmap_drawing_area == CNMSNULL ) || ( fr_current_gc_drawing_area == CNMSNULL ) ){
		DBGMSG( "[CnmsFRToneCurveDraw] Arg Pointer is NULL. widget[%p], pixmap[%p], gc[%p]\n", widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area );
		return;
	}

	/* Drawing Graph Frame		*/
	CnmsColAdjDrawAreaBaseDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area );
	
	/* Set Graph ToneCurve		*/
	switch( color )
	{
		case CNMS_CA_COLORGROUP_COLOR:
			switch( channel )
			{
				case CNMS_CA_CHANNEL_MASTER:
					CnmsColAdjDrawAreaHistogramDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_master.hist, CNMS_CA_COLOR_BLACK );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_red.curve, CNMS_CA_COLOR_RED );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_green.curve, CNMS_CA_COLOR_GREEN );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_blue.curve, CNMS_CA_COLOR_BLUE );
					break;
				
				case CNMS_CA_CHANNEL_RED:
					CnmsColAdjDrawAreaHistogramDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_red.hist, CNMS_CA_COLOR_BLACK );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_green.curve, CNMS_CA_COLOR_GREEN );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_blue.curve, CNMS_CA_COLOR_BLUE );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_red.curve, CNMS_CA_COLOR_RED );
					break;
				
				case CNMS_CA_CHANNEL_GREEN:
					CnmsColAdjDrawAreaHistogramDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_green.hist, CNMS_CA_COLOR_BLACK );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_red.curve, CNMS_CA_COLOR_RED );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_blue.curve, CNMS_CA_COLOR_BLUE );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_green.curve, CNMS_CA_COLOR_GREEN );
					break;
				
				case CNMS_CA_CHANNEL_BLUE:
					CnmsColAdjDrawAreaHistogramDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_blue.hist, CNMS_CA_COLOR_BLACK );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_red.curve, CNMS_CA_COLOR_RED );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_green.curve, CNMS_CA_COLOR_GREEN );
					CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_blue.curve, CNMS_CA_COLOR_BLUE );
					break;
				
				default:
					DBGMSG( "[CnmsFRToneCurveDraw] Arg value is invalid. channel[%d]\n", channel );
					break;
			}
			break;
			
		case CNMS_CA_COLORGROUP_GRAY:
					CnmsColAdjDrawAreaHistogramDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_gray.hist, CNMS_CA_COLOR_BLACK );
			CnmsColAdjDrawAreaCurveDraw( widget, fr_pixmap_drawing_area, fr_current_gc_drawing_area, fr_gray.curve, CNMS_CA_COLOR_GRAY );
			break;
			
		default:
			DBGMSG( "[CnmsFRToneCurveDraw] Arg value is invalid. color[%d]\n", color );
			break;
	}

	/* Drawing Pixmap			*/
	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		fr_pixmap_drawing_area, 0, 0, 0, 0, widget->allocation.width, widget->allocation.height);
		
	return;
}


/***   Set ToneCurve & Data depending on Channel   ***/
static CNMSVoid CnmsFRSetChannelData(
						CNMSFRValue			*current				)
{
	if( current == CNMSNULL ){
		DBGMSG( "[CnmsFRSetChannelData] Arg Pointer is NULL. current[%p]\n", current );
		return;
	}

	/* Set ToneCurve	*/
	fr_current = current;
	CnmsFRToneCurveDraw( fr_drawarea, ca_current_colorgroup_num, fr_current_channel );
	
	return;
}

#endif	/* _COLORADJUST_FR_C_ */



